@@ -1,5 +1,4 @@
 """Functionality for converting a Format 2 workflow into a standard Galaxy workflow."""
-from __future__ import print_function
 
 import argparse
 import copy
@@ -8,14 +7,14 @@ import logging
 import os
 import sys
 import uuid
-from collections import OrderedDict
-from typing import Dict, Optional
+from typing import Any, Dict, Optional
 
 from ._labels import Labels
 from .model import (
     convert_dict_to_id_list_if_needed,
     ensure_step_position,
-    inputs_as_steps,
+    inputs_as_native_steps,
+    with_step_ids,
 )
 from .yaml import ordered_load
 
@@ -93,13 +92,13 @@ def rename_arg(argument):
 def clean_connection(value):
     if value and "#" in value and SUPPORT_LEGACY_CONNECTIONS:
         # Hope these are just used by Galaxy testing workflows and such, and not in production workflows.
-        log.warn("Legacy workflow syntax for connections [%s] will not be supported in the future" % value)
+        log.warn(f"Legacy workflow syntax for connections [{value}] will not be supported in the future")
         value = value.replace("#", "/", 1)
     else:
         return value
 
 
-class ImportOptions(object):
+class ImportOptions:
 
     def __init__(self):
         self.deduplicate_subworkflows = False
@@ -129,7 +128,7 @@ def python_to_workflow(as_python, galaxy_interface, workflow_directory=None, imp
     if conversion_context.import_options.deduplicate_subworkflows:
         # TODO: import only required workflows...
         # TODO: dag sort these...
-        subworkflows = OrderedDict()
+        subworkflows = {}
         for graph_id, subworkflow_content in conversion_context.graph_ids.items():
             if graph_id == "main":
                 continue
@@ -149,13 +148,15 @@ def steps_as_list(format2_workflow: dict, add_ids: bool = False, inputs_offset:
 
     Add keys as labels instead of IDs. Why am I doing this?
     """
+    if "steps" not in format2_workflow:
+        raise Exception(f"No 'steps' key in dict, keys are {format2_workflow.keys()}")
     steps = format2_workflow["steps"]
     steps = convert_dict_to_id_list_if_needed(steps, add_label=True, mutate=mutate)
     if add_ids:
         if mutate:
             _append_step_id_to_step_list_elements(steps, inputs_offset=inputs_offset)
         else:
-            steps = _with_step_ids(steps, inputs_offset=inputs_offset)
+            steps = with_step_ids(steps, inputs_offset=inputs_offset)
     return steps
 
 
@@ -167,18 +168,6 @@ def _append_step_id_to_step_list_elements(steps: list, inputs_offset: int = 0):
         assert step["id"] is not None
 
 
-def _with_step_ids(steps: list, inputs_offset: int = 0):
-    assert isinstance(steps, list)
-    new_steps = []
-    for i, step in enumerate(steps):
-        if "id" not in step:
-            step = step.copy()
-            step["id"] = i + inputs_offset
-        assert step["id"] is not None
-        new_steps.append(step)
-    return new_steps
-
-
 def _python_to_workflow(as_python, conversion_context):
 
     if "class" not in as_python:
@@ -193,7 +182,7 @@ def _python_to_workflow(as_python, conversion_context):
     _ensure_defaults(as_python, {
         "a_galaxy_workflow": "true",
         "format-version": "0.1",
-        "name": "Workflow",
+        "name": as_python.pop("label", "Workflow"),
         "uuid": str(uuid.uuid4()),
     })
     _populate_annotation(as_python)
@@ -204,7 +193,7 @@ def _python_to_workflow(as_python, conversion_context):
 
     if isinstance(steps, list):
         _append_step_id_to_step_list_elements(steps)
-        steps_as_dict = OrderedDict()
+        steps_as_dict: Dict[str, Any] = {}
         for i, step in enumerate(steps):
             steps_as_dict[str(i)] = step
             if "label" in step:
@@ -238,9 +227,9 @@ def _python_to_workflow(as_python, conversion_context):
         step_type = step.get("type", "tool")
         step_type = STEP_TYPE_ALIASES.get(step_type, step_type)
         if step_type not in STEP_TYPES:
-            raise Exception("Unknown step type encountered %s" % step_type)
+            raise Exception(f"Unknown step type encountered {step_type}")
         step["type"] = step_type
-        eval("transform_%s" % step_type)(conversion_context, step)
+        eval(f"transform_{step_type}")(conversion_context, step)
 
     outputs = as_python.pop("outputs", [])
     outputs = convert_dict_to_id_list_if_needed(outputs)
@@ -308,7 +297,7 @@ def convert_inputs_to_steps(workflow_dict: dict, steps: list):
     if "inputs" not in workflow_dict:
         return
 
-    input_steps = inputs_as_steps(workflow_dict)
+    input_steps = inputs_as_native_steps(workflow_dict)
     workflow_dict.pop("inputs")
     for i, new_step in enumerate(input_steps):
         steps.insert(i, new_step)
@@ -398,6 +387,10 @@ def transform_pause(context, step, default_name="Pause for dataset review"):
 
 
 def transform_subworkflow(context, step):
+    if "when" in step and "source" in step["when"]:
+        step_id, output_name = context.step_output(step["when"]["source"])
+        step["when"]["source"] = {"id": step_id, "output_name": output_name}
+
     _populate_annotation(step)
 
     _ensure_inputs_connections(step)
@@ -521,7 +514,7 @@ def run_tool_to_step(conversion_context, step, run_action):
     step["tool_uuid"] = tool_description.get("uuid")
 
 
-class BaseConversionContext(object):
+class BaseConversionContext:
 
     def __init__(self):
         self.labels = {}
@@ -529,10 +522,10 @@ class BaseConversionContext(object):
 
     def step_id(self, label_or_id):
         if label_or_id in self.labels:
-            id = self.labels[label_or_id]
+            id_ = self.labels[label_or_id]
         else:
-            id = label_or_id
-        return int(id)
+            id_ = label_or_id
+        return int(id_)
 
     def step_output(self, value):
         value_parts = str(value).split("/")
@@ -568,7 +561,7 @@ class BaseConversionContext(object):
 
             run_action_path = run_action["@import"]
             runnable_path = os.path.join(self.workflow_directory, run_action_path)
-            with open(runnable_path, "r") as f:
+            with open(runnable_path) as f:
                 runnable_description = ordered_load(f)
                 run_action = runnable_description
 
@@ -581,10 +574,10 @@ class BaseConversionContext(object):
 class ConversionContext(BaseConversionContext):
 
     def __init__(self, galaxy_interface, workflow_directory, import_options: Optional[ImportOptions] = None):
-        super(ConversionContext, self).__init__()
+        super().__init__()
         self.import_options = import_options or ImportOptions()
-        self.graph_ids = OrderedDict()  # type: Dict
-        self.graph_id_subworkflow_conversion_contexts = {}  # type: Dict
+        self.graph_ids: Dict[str, Any] = {}
+        self.graph_id_subworkflow_conversion_contexts: Dict[str, Any] = {}
         self.workflow_directory = workflow_directory
         self.galaxy_interface = galaxy_interface
 
@@ -604,7 +597,7 @@ class ConversionContext(BaseConversionContext):
 class SubworkflowConversionContext(BaseConversionContext):
 
     def __init__(self, parent_context):
-        super(SubworkflowConversionContext, self).__init__()
+        super().__init__()
         self.parent_context = parent_context
 
     @property
@@ -627,7 +620,7 @@ class SubworkflowConversionContext(BaseConversionContext):
         return self.parent_context.get_subworkflow_conversion_context_graph(graph_id)
 
 
-def _action(type, name, arguments={}):
+def _action(type, name, arguments):
     return {
         "action_arguments": arguments,
         "action_type": type,
@@ -641,7 +634,7 @@ def _is_link(value):
 
 def _join_prefix(prefix, key):
     if prefix:
-        new_key = "%s|%s" % (prefix, key)
+        new_key = f"{prefix}|{key}"
     else:
         new_key = key
     return new_key
@@ -666,7 +659,7 @@ def _init_connect_dict(step):
             elif isinstance(value, dict) and 'default' in value:
                 continue
             elif isinstance(value, dict):
-                raise KeyError('step input must define either source or default %s' % value)
+                raise KeyError(f'step input must define either source or default {value}')
             connect[key] = [value]
             connection_keys.add(key)
 
@@ -692,13 +685,17 @@ def _populate_input_connections(context, step, connect):
             if not isinstance(value, dict):
                 if key == "$step":
                     value += "/__NO_INPUT_OUTPUT_NAME__"
-                id, output_name = context.step_output(value)
-                value = {"id": id, "output_name": output_name}
-                if is_subworkflow_step:
-                    subworkflow_conversion_context = context.get_subworkflow_conversion_context(step)
-                    input_subworkflow_step_id = subworkflow_conversion_context.step_id(key)
-                    value["input_subworkflow_step_id"] = input_subworkflow_step_id
-            input_connection_value.append(value)
+                if not isinstance(value, list):
+                    value = [value]
+                for source in value:
+                    step_id, output_name = context.step_output(source)
+                    source = {"id": step_id, "output_name": output_name}
+                    if is_subworkflow_step:
+                        subworkflow_conversion_context = context.get_subworkflow_conversion_context(step)
+                        if key in subworkflow_conversion_context.labels:
+                            input_subworkflow_step_id = subworkflow_conversion_context.step_id(key)
+                            source["input_subworkflow_step_id"] = input_subworkflow_step_id
+                    input_connection_value.append(source)
         if key == "$step":
             key = "__NO_INPUT_OUTPUT_NAME__"
         input_connections[key] = input_connection_value
@@ -740,7 +737,7 @@ def main(argv=None):
     workflow_directory = os.path.abspath(format2_path)
     galaxy_interface = None
 
-    with open(format2_path, "r") as f:
+    with open(format2_path) as f:
         has_workflow = ordered_load(f)
 
     output = python_to_workflow(has_workflow, galaxy_interface=galaxy_interface, workflow_directory=workflow_directory)
